# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006-2007 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package Wanha::HTTP;

use strict;
use warnings FATAL => 'all';
use diagnostics;

use Wanha::Config;


my %get_val;
my %post_val;

my @cookies;
my %cookie_value;
my %cookie_expire;
my $redirect_location;


BEGIN {
	if (defined($ENV{"QUERY_STRING"})) {
		foreach $_ (split(/&/, $ENV{"QUERY_STRING"})) {
			my $key;
			($key, $_) = /(.*?)=(.*)/;
			(defined($key)) || next;
			tr/+/ /;
			s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
			$get_val{$key} = $_;
		}
	}
	
	undef $/;
	my $post_data = <STDIN>;
	if (defined($post_data)) {
		foreach $_ (split(/&/, $post_data)) {
			my $key;
			($key, $_) = /(.*?)=(.*)/;
			(defined($key)) || next;
			tr/+/ /;
			s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
			$post_val{$key} = $_;
		}
	}
}



=item get
Get GET value or return default value if key is not set.

0: key
1: default
r: value
=cut
sub get($$) {
	my ($key, $def) = @_;
	my $val = $get_val{$key};
	return defined($val) ? $val : $def;
}


=item get_numeric
Get numeric GET value or return default value if key is not set.

0: key
1: default
r: value
=cut
sub get_numeric($$)
{
	my ($key, $def) = @_;
	my $val = &get($key, $def);
	$val = &numeric($val) if (defined($val));
	return (! defined($val) || $val eq "") ? $def : $val;
}


=item post
Get POST value or return default value if key is not set.

0: key
1: default
r: value
=cut
sub post($$) {
	my ($key, $def) = @_;
	my $val = $post_val{"$key"};
	return defined($val) ? $val : $def;
}


=item post
Get numeric POST value or return default value if key is not set.

0: key
1: default
r: value
=cut
sub post_numeric($$) {
	my ($key, $def) = @_;
	my $val = &post($key, $def);
	$val = &numeric($val) if (defined($val));
	return ($val eq "") ? $def : $val;
}


=item get_cookies
Get cookies

r: cookies
=cut
sub get_cookies() {
	my @cookies;
	my $t = $ENV{"HTTP_COOKIE"};
	(defined($t)) || return ();

	foreach $_ (split ('; ', $t)) {
		push(@cookies, $_);
	}
	return @cookies;
}


=item set_cookie
Set cookies to be sent

@: (name, value, age)
=cut
sub set_cookie(@) {
	while ($#_ >= 0) {
		my $name = shift;
		my $value = shift;
		my $age = shift;
		push(@cookies, $name);
		$cookie_value{$name} = $value;
		$cookie_expire{$name} = $age;
	}
}


=item print_header
Print HTTP header.
=cut
sub print_header() {
	my ($domain, $path) = $Wanha::Config::base_url =~ /.*?\/\/(.*?)(\/.*)/;
	$domain = ".$domain";

	print "Content-Type: text/html; charset=UTF-8\n";
	print "Cache-Control: private\n";
	foreach my $key (@cookies) {
		printf "Set-Cookie: %s=%s; Path=%s; Domain=%s; Max-Age=%s\n",
				$key, $cookie_value{$key},
				$path, $domain, $cookie_expire{$key};
	}
	(defined($redirect_location)) && print "Location: $redirect_location\n";
	print "\n";
}


=item redirect
Set HTTP redirect location.

0: location
=cut
sub redirect($) {
	($redirect_location) = @_;
	$redirect_location =~ s/&amp;/\&/g;
}


=item get_ip
Get client IP address

r: ip
=cut
sub get_ip() {
	return $ENV{"REMOTE_ADDR"};
}


=item numeric
Get numeric values of a string only.

0: string
r: value
=cut
sub numeric($) {
	$_ = shift;
	s/[^0-9\-\.]//g;
	if (/^-/) {
		s/-//g;
		s/^/-/;
	} else {
		s/-//g;
	}
	s/\./+/;
	s/\.//g;
	s/\+/./;
	$_ = "" if (/\./);
	return $_;
}


=item dump_post
Dump POST data
=cut
sub dump_post() {
	print "<pre>\n";
	while (my ($key, $val) = each %post_val) {
		print "$key = $val\n";
	}
	print "</pre>\n";
}



return 1;
