#! /usr/bin/perl
#
# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006-2007 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use warnings FATAL => 'all';
use diagnostics;

use Wanha::Chan;
use Wanha::Nick;

use Wanha::Session;
use Wanha::URLList;
use Wanha::Link;

use Wanha::HTML;
use Wanha::HTTP;
use Wanha::DB;

use Wanha::Common;
use Wanha::Lang;


my $max_days = 90;
my $top_n = 10;


=item main
Main function
=cut
sub main() {
	# Update cookie
	my $sid = &Wanha::Session::get_id();

	&Wanha::HTTP::print_header();
	&Wanha::HTML::print_header();

	my ($chan_id, $chan_name) = &Wanha::Session::guess_channel();
	&Wanha::Session::save_channel($sid, $chan_id);
	&Wanha::Session::print_links(undef, &Wanha::Link::stat($chan_id));

	&Wanha::Lang::printf("<h1><&stat-title&></h1>\n");

	my $url = &Wanha::HTTP::get("str", "");	# (part of) URL to search

	my $url_pos = &Wanha::HTTP::get("pos", "");	# bit: 1: begin, 2: end
	$url_pos = 0 if (! ($url_pos =~ /./));

#	my $url_strip = &HTTP::get("url_s", 1);	# strip www.

	my $age_max = &Wanha::HTTP::get_numeric("max_a", 90);
	$age_max = 1 if ($age_max < 1);

	my $age_min = &Wanha::HTTP::get_numeric("min_a", 0);

	# 0: urls per user (user top n)
	# 1: urls per hour per user (hour 0-23)
	# 2: domains per user (domain top n)
	my $stat = &Wanha::HTTP::get_numeric("r", 0);

	if ($age_max - $age_min > $max_days) {
		$age_min = $age_max - $max_days;
	}

	my $r;
	if ($url =~ /./ && defined($chan_id)) {
		if ($stat == 0) {
			$r = &user_top($chan_id, $url, $url_pos,
					$age_max, $age_min);
		}
	}

	&query_page($chan_name, $url, $age_max, $age_min);

	&Wanha::HTML::print_footer(&Wanha::Common::process_time());
}


=item query_page
Print statistics query form

0: chan
1: url
2: age_max
3: age_min
=cut
sub query_page($$$$) {
	my ($d_chan, $d_url, $d_age_max, $d_age_min) = @_;

	my $l_chan = &Wanha::Lang::get("col-channel");
	my $l_url = &Wanha::Lang::get("col-url");
	my $l_age = &Wanha::Lang::get("col-age");
	my $l_max = &Wanha::Lang::get("txt-age-max");
	my $l_min = &Wanha::Lang::get("txt-age-min");
	my $l_days = &Wanha::Lang::get("txt-days");
	my $l_search = &Wanha::Lang::get("txt-search");

	if (! defined($d_chan) || ! ($d_chan =~ /./)) {
		$d_chan = &Wanha::Lang::get("txt-any");
	}
	$d_url = "" if (! defined($d_url));
	$d_age_max = $max_days if (! defined($d_age_max));
	$d_age_min = 0 if (! defined($d_age_min));

	&Wanha::Lang::printf("<h2><&stat-search&></h2>\n\n");

	my $self = &Wanha::Link::stat();
	$self =~ s/.*\///;
	print <<EOF;
<form action="$self" method="get">
<table class="search">
	<tr>
		<th>$l_chan</th>
		<td colspan="2"><input type="text" name="chan" size="40"
				value="$d_chan" /></td>
	</tr>
	<tr>
		<th>$l_url</th>
		<td colspan="2"><input type="text" name="str" size="40"
				value="$d_url" /></td>
	</tr>
	<tr>
		<th rowspan="2">$l_age</th>
		<td>$l_max</td>
		<td class="max"><input type="text" name="max_a" size="4"
				value="$d_age_max" />$l_days</td>
	</tr>
	<tr>
		<td>$l_min</td>
		<td class="max"><input type="text" name="min_a" size="4"
					value="$d_age_min" />$l_days</td>
	</tr>
	<tr>
		<th></th>
		<td><input type="submit" value="$l_search" /></td>
	</tr>
</table>
</form>
EOF
}


=item user_top
Print user top

0: chan
1: url
2: url_pos
3: age_max
4: age_min
=cut
sub user_top($$$$$) {
	my ($chan, $url_str, $url_pos, $age_max, $age_min) = @_;
	my ($query, $result);

	&Wanha::Lang::printf("<p><&stat-top-n-users&> ", $top_n);

	if ($url_pos == 3) {
		&Wanha::Lang::printf("<&stat-urls-exactly-like&>", $url_str);
	} else {
		my $url_match;
		if ($url_pos == 0) {
			$url_match = "*$url_str*";
		} elsif ($url_pos == 1) {
			$url_match = "$url_str*";
		} elsif ($url_pos == 2) {
			$url_match = "*$url_str";
		}
		&Wanha::Lang::printf("<&stat-urls-like&>", $url_match);
	}

	print " ";
	if ($chan == -1) {
		&Wanha::Lang::printf("<&stat-on-any-channel&>");
	} else {
		my $chan_name = &Wanha::Chan::get_name($chan);
		&Wanha::Lang::printf("<&stat-on-channel&>", $chan_name);
	}

	print " ";
	&Wanha::Lang::printf("<&stat-aged-max&> ", $age_max);
	if ($age_min != 0) {
		&Wanha::Lang::printf(" <&stat-aged-min&>", $age_min);
	}
	&Wanha::Lang::printf("<&stat-fullstop&></p>\n\n");


	$query = "SELECT id, nick FROM urls WHERE ";
	$query .= "chan = &da& AND " if ($chan != -1);
	$query .= "CURRENT_TIMESTAMP - time < INTERVAL '&db& days' AND ";
	$query .= "CURRENT_TIMESTAMP - time >= INTERVAL '&dc& days' AND "
			if ($age_min > 0);
	$query .= "url ILIKE &sd&";

	$query = &Wanha::DB::prepare($query);
	$query = &Wanha::DB::set($query, "a", $chan) if ($chan != -1);
	$query = &Wanha::DB::set($query, "b", $age_max);
	$query = &Wanha::DB::set($query, "c", $age_min) if ($age_min > 0);
	$query = &Wanha::DB::set($query, "d", "\%$url_str\%");

	$result = &Wanha::DB::query($query);
	if (&Wanha::DB::row_count($result) == 0) {
		&Wanha::Lang::printf("<p><&stat-no-matches&></p>\n\n");
		return 0;
	}

	my $url_count = 0;
	my %user_url;
	while (my ($url, $user) = &Wanha::DB::get_row($result)) {
		($url_pos == 1 && ! ($url =~ /^$url_str/i)) && next;
		($url_pos == 2 && ! ($url =~ /$url_str$/i)) && next;
		($url_pos == 3 && ! ($url =~ /^$url_str$/i)) && next;

		$user_url{$user}++;
		$url_count++;
	}

	my @top_user;
	while (my ($user, $count) = each %user_url) {
		push(@top_user, "$count\t$user");
	}
	@top_user = sort Wanha::Common::compare_rank_desc @top_user;
	my $max_i = &Wanha::Common::min($top_n, $#top_user + 1);

	my @user_ids;
	for (my $i = 0; $i < $max_i; $i++) {
		my ($id) = $top_user[$i] =~ /\t(\d+)/;
		push(@user_ids, $id);
	}
	my %users = &Wanha::Nick::get_names(@user_ids);

	my $l_nick = &Wanha::Lang::get("col-nick");
	my $l_count = &Wanha::Lang::get("col-times-pasted");
	my $l_of_all = &Wanha::Lang::get("col-share-of-all");
	print <<EOF;
<table class="pasters">
	<tr>
		<th>$l_nick</th>
		<th>$l_count</th>
		<th>$l_of_all</th>
	</tr>
EOF
	my $d_sum = 0;
	for (my $i = 0; $i < $max_i; $i++) {
		my ($top_count, $top_user) = split(/\t/, $top_user[$i], 2);
		my $p_user = $users{$top_user};
		my $l_user = &Wanha::Link::user($top_user);
		my $p_count = $top_count;
		my $p_perc = sprintf("%.1f", $p_count / $url_count * 100);
		my $class = $i % 2 == 1 ? " class=\"zebra\"" : "";
		$d_sum += $p_count;
		print <<EOF;
	<tr$class>
		<td><a href="$l_user">$p_user</a></td>
		<td>$p_count</td>
		<td>$p_perc%</td>
	</tr>
EOF
	}
	my $l_total = &Wanha::Lang::get("col-total");
	my $d_perc = sprintf("%.1f", $d_sum / $url_count * 100);
	print <<EOF;
	<tr class="sum">
		<td>$l_total</th>
		<td>$d_sum</td>
		<td>$d_perc%</td>
	</tr>
</table>
EOF

	return 1;
}


&main();
exit(0);
