#! /usr/bin/perl
#
# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006-2009 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use warnings FATAL => 'all';
use diagnostics;

use Wanha::HTTP;
use Wanha::HTML;
use Wanha::DB;
use Wanha::Session;
use Wanha::Link;
use Wanha::URL;
use Wanha::URLList;
use Wanha::Common;


=item get_latest
Get latest URLs

@: channels
r: array of hash (url_id, url_url, url_count, url_date, url_paste,
		chan_id, chan_name)
=cut
sub get_latest(@) {
	my @chans = @_;
	my @ret;

	# chans are ignored for now

	my $date_fmt = "hh24:mi:ss dd Mon yyyy";
	my $w_date = &Wanha::DB::get_date_fmt("w.time", "w_time", $date_fmt);
	my $u_date = &Wanha::DB::get_date_fmt("u.time", "u_time", $date_fmt);

	my $query = &Wanha::DB::prepare(
		"SELECT u.id, u.url, c.id, c.chan, u.count, " .
		"$w_date, $u_date, " .
		"EXTRACT(EPOCH FROM w.time) - " .
			"EXTRACT(EPOCH FROM u.time) AS age " .
		"FROM chans as c, wanha as w, urls as u, nicks as n " .
		"WHERE w.url = u.id AND u.chan = c.id AND w.repaste = n.id " .
		"ORDER BY w.time DESC LIMIT &da&");

	$query = &Wanha::DB::set($query, "a", 25);
	my $result = &Wanha::DB::query($query);
	(&Wanha::DB::row_count($result) == 0) && return ();

	while (my ($url_id, $url_url, $chan_id, $chan_name, $url_count,
			$wanha_date, $url_date, $url_age) =
			&Wanha::DB::get_row($result)) {
		my %t = (
			'url_id' => $url_id,
			'url_url' => $url_url,
			'url_count' => $url_count,
			'url_date' => $url_date,
			'url_re_date' => $wanha_date,
			'url_age' => $url_age,
			'url_count' => $url_count,
			'chan_id' => $chan_id,
			'chan_name' => $chan_name
			);
		push(@ret, \%t);
	}
	return @ret;
}


=item latest
Print latest URLs
=cut
sub latest() {
	my @urls = &get_latest();

	if ($#urls == -1) {
		&Wanha::Lang::printf("<h1><&no-wanhas-pasted&></h1>\n");
		return;
	}

	&Wanha::Lang::printf("<h1><&w-latest&></h1>\n");
	&Wanha::URLList::print(\@urls, "url", "age", "count", "chan", "redate");
}


=item main
Main function
=cut
sub main() {
	&Wanha::HTTP::print_header();
	&Wanha::HTML::print_header();

	&Wanha::Session::print_links();

	&latest();

	&Wanha::HTML::print_footer(&Wanha::Common::process_time());
}


&main();
